@@ -2,9 +2,9 @@
 #
 #	XS.pm : perl function definition for Term::ReadLine::Gnu
 #
-#	$Id: XS.pm,v 1.18 2002-03-29 23:12:28-05 hiroo Exp $
+#	$Id: XS.pm,v 1.23 2008-02-08 00:43:46+09 hiroo Exp $
 #
-#	Copyright (c) 2001 Hiroo Hayashi.  All rights reserved.
+#	Copyright (c) 2008 Hiroo Hayashi.  All rights reserved.
 #
 #	This program is free software; you can redistribute it and/or
 #	modify it under the same terms as Perl itself.
@@ -15,6 +15,9 @@ use Carp;
 use strict;
 use AutoLoader 'AUTOLOAD';
 
+use vars qw($VERSION);
+$VERSION='1.17';	# added for CPAN
+
 # make aliases
 use vars qw(%Attribs);
 *Attribs = \%Term::ReadLine::Gnu::Attribs;
@@ -87,6 +90,7 @@ sub rl_copy_keymap ($)    { return _rl_copy_keymap(_str2map($_[0])); }
 sub rl_discard_keymap ($) { return _rl_discard_keymap(_str2map($_[0])); }
 sub rl_set_keymap ($)     { return _rl_set_keymap(_str2map($_[0])); }
 
+# rl_bind_key
 sub rl_bind_key ($$;$) {
     if (defined $_[2]) {
 	return _rl_bind_key($_[0], _str2fn($_[1]), _str2map($_[2]));
@@ -95,6 +99,21 @@ sub rl_bind_key ($$;$) {
     }
 }
 
+# rl_bind_key_if_unbound
+sub rl_bind_key_if_unbound ($$;$) {
+    my ($version) = $Attribs{library_version}
+	=~ /(\d+\.\d+)/;
+    if ($version < 5.0) {
+	carp "rl_bind_key_if_unbound() is not supported.  Ignored\n";
+	return;
+    }
+    if (defined $_[2]) {
+	return _rl_bind_key_if_unbound($_[0], _str2fn($_[1]), _str2map($_[2]));
+    } else {
+	return _rl_bind_key_if_unbound($_[0], _str2fn($_[1]));
+    }
+}
+
 # rl_unbind_key
 sub unbind_key ($;$) {
     if (defined $_[1]) {
@@ -135,6 +154,21 @@ sub unbind_command ($;$) {
     }
 }
 
+# rl_bind_keyseq
+sub rl_bind_keyseq ($$;$) {
+    my ($version) = $Attribs{library_version}
+	=~ /(\d+\.\d+)/;
+    if ($version < 5.0) {
+	carp "rl_bind_keyseq() is not supported.  Ignored\n";
+	return;
+    }
+    if (defined $_[2]) {
+	return _rl_bind_keyseq($_[0], _str2fn($_[1]), _str2map($_[2]));
+    } else {
+	return _rl_bind_keyseq($_[0], _str2fn($_[1]));
+    }
+}
+
 sub rl_set_key ($$;$) {
     my ($version) = $Attribs{library_version}
 	=~ /(\d+\.\d+)/;
@@ -149,6 +183,21 @@ sub rl_set_key ($$;$) {
     }
 }
 
+# rl_bind_keyseq_if_unbound
+sub rl_bind_keyseq_if_unbound ($$;$) {
+    my ($version) = $Attribs{library_version}
+	=~ /(\d+\.\d+)/;
+    if ($version < 5.0) {
+	carp "rl_bind_keyseq_if_unbound() is not supported.  Ignored\n";
+	return;
+    }
+    if (defined $_[2]) {
+	return _rl_bind_keyseq_if_unbound($_[0], _str2fn($_[1]), _str2map($_[2]));
+    } else {
+	return _rl_bind_keyseq_if_unbound($_[0], _str2fn($_[1]));
+    }
+}
+
 sub rl_macro_bind ($$;$) {
     my ($version) = $Attribs{library_version}
 	=~ /(\d+\.\d+)/;
@@ -182,7 +231,7 @@ sub rl_generic_bind ($$$;$) {
 	carp("Term::ReadLine::Gnu::rl_generic_bind: invalid \`type\'\n");
     }
 }
-	    
+
 sub rl_call_function ($;$$) {
     if (defined $_[2]) {
 	return _rl_call_function(_str2fn($_[0]), $_[1], $_[2]);
@@ -212,6 +261,12 @@ sub rl_add_funmap_entry ($$) {
 }
 
 sub rl_tty_set_default_bindings (;$) {
+    my ($version) = $Attribs{library_version}
+	=~ /(\d+\.\d+)/;
+    if ($version < 4.2) {
+	carp "rl_tty_set_default_bindings() is not supported.  Ignored\n";
+	return;
+    }
     if (defined $_[0]) {
 	return _rl_tty_set_defaut_bindings(_str2map($_[1]));
     } else {
@@ -219,12 +274,37 @@ sub rl_tty_set_default_bindings (;$) {
     }
 }
 
+sub rl_tty_unset_default_bindings (;$) {
+    my ($version) = $Attribs{library_version}
+	=~ /(\d+\.\d+)/;
+    if ($version < 5.0) {
+	carp "rl_tty_unset_default_bindings() is not supported.  Ignored\n";
+	return;
+    }
+    if (defined $_[0]) {
+	return _rl_tty_unset_defaut_bindings(_str2map($_[1]));
+    } else {
+	return _rl_tty_unset_defaut_bindings();
+    }
+}
+
 sub rl_message {
     my $fmt = shift;
     my $line = sprintf($fmt, @_);
     _rl_message($line);
 }
 
+sub rl_completion_mode {
+    # libreadline.* in Debian GNU/Linux 2.0 tells wrong value as '2.1-bash'
+    my ($version) = $Attribs{library_version}
+	=~ /(\d+\.\d+)/;
+    if ($version < 4.3) {
+	carp "rl_completion_mode() is not supported.  Ignored\n";
+	return;
+    }
+    return _rl_completion_mode(_str2fn($_[0]));
+}
+
 #
 #	for compatibility with Term::ReadLine::Perl
 #
@@ -409,11 +489,15 @@ sub Tk_getc {
 #	$line = $t->readline("password> ");
 sub shadow_redisplay {
     @_tstrs = _tgetstrs() unless $_tstrs_init;
+    # remove prompt start/end mark from prompt string
+    my $prompt = $Attribs{prompt}; my $s;
+    $s = Term::ReadLine::Gnu::RL_PROMPT_START_IGNORE; $prompt =~ s/$s//g;
+    $s = Term::ReadLine::Gnu::RL_PROMPT_END_IGNORE;   $prompt =~ s/$s//g;
     my $OUT = $Attribs{outstream};
     my $oldfh = select($OUT); $| = 1; select($oldfh);
     print $OUT ($_tstrs[0],	# carriage return
 		$_tstrs[1],	# clear to EOL
-		$Attribs{prompt}, '*' x length($Attribs{line_buffer}));
+		$prompt, '*' x length($Attribs{line_buffer}));
     print $OUT ($_tstrs[2]	# cursor left
 		x (length($Attribs{line_buffer}) - $Attribs{point}));
     $oldfh = select($OUT); $| = 0; select($oldfh);
@@ -443,13 +527,13 @@ sub _ch_wrapper {
 	    ($result, $line) = history_expand($line);
 	    my $outstream = $Attribs{outstream};
 	    print $outstream "$line\n" if ($result);
-	    
+
 	    # return without adding line into history
 	    if ($result < 0 || $result == 2) {
 		return '';	# don't return `undef' which means EOF.
 	    }
 	}
-	
+
 	# add to history buffer
 	add_history($line) 
 	    if ($Attribs{MinLength} > 0
@@ -463,7 +547,7 @@ sub _ch_wrapper {
 #
 sub list_completion_function ( $$ ) {
     my($text, $state) = @_;
-    
+
     $_i = $state ? $_i + 1 : 0;	# clear counter at the first call
     my $cw = $Attribs{completion_word};
     for (; $_i <= $#{$cw}; $_i++) {
@@ -481,7 +565,7 @@ sub _trp_completion_function ( $$ ) {
 
     my $cf;
     return undef unless defined ($cf = $Attribs{completion_function});
-    
+
     if ($state) {
 	$_i++;
     } else {
@@ -494,7 +578,7 @@ sub _trp_completion_function ( $$ ) {
 	# @_matches = undef
 	return undef unless defined $_matches[0];
     }
-    
+
     for (; $_i <= $#_matches; $_i++) {
 	return $_matches[$_i] if ($_matches[$_i] =~ /^\Q$text/);
     }
